@@ -1,16 +1,25 @@
+def warn(*args, **kwargs):
+    pass
+
+
+import warnings
+
+warnings.warn = warn
+
 from abc import abstractmethod, ABC
 from typing import Tuple, Optional, Callable, Any, List, Dict, Union
-from itertools import compress
-
 from scipy.stats import PearsonRConstantInputWarning
 from sklearn.preprocessing import StandardScaler
+import sklearn.gaussian_process as gp
 import numpy as np
 import scipy as sp
 import time
 from pathlib import Path
+import river as rv
 
 from movado.estimator import Estimator
 from movado.mab_handler import MabHandler
+from movado.chained_estimator import ChainedEstimator
 
 is_call_exact: List[bool] = []  # i-th element True if i-th call is exact
 
@@ -34,6 +43,32 @@ class Controller(ABC):
         self._estimator: Estimator = estimator
         self._debug = debug
         self._self_exact = self_exact
+        self.__model = rv.preprocessing.StandardScaler()
+        self.__model |= rv.feature_extraction.RBFSampler()
+        self.__model |= rv.linear_model.LinearRegression()
+        models = rv.utils.expand_param_grid(
+            self.__model,
+            {
+                "LinearRegression": {
+                    "l2": [1e-3, 1e-1, 1, 10],
+                },
+                "RBFSampler": {"gamma": [1e-3, 1e-1, 1, 10]},
+            },
+        )
+        self.__time_model = rv.expert.SuccessiveHalvingRegressor(
+            models=models,
+            metric=rv.metrics.RMSE(),
+            budget=2,
+            eta=2000,
+            verbose=True,
+        )
+        self.__error_model = rv.expert.SuccessiveHalvingRegressor(
+            models=models,
+            metric=rv.metrics.RMSE(),
+            budget=2,
+            eta=2000,
+            verbose=True,
+        )
         if self._debug:
             Path("movado_debug").mkdir(exist_ok=True)
             self._controller_debug = "movado_debug/controller.csv"
@@ -78,7 +113,7 @@ class Controller(ABC):
                     forced_action=mab_forced_action,
                 ),
                 mab_weight.learn(
-                    cost=self.get_time_error_correlation(),
+                    cost=self.compute_first_order_sensitivity_index(),
                     context=self._compute_weighting_context(
                         self.get_mab().get_mean_cost()
                     ),
@@ -199,18 +234,19 @@ class Controller(ABC):
     def write_debug(self, debug_info: Dict[str, Any]):
         pass
 
+    def __reshape(self, to_reshape):
+        if len(self.__time_dataset) > 1:
+            return np.reshape(to_reshape, (-1, 1))
+        else:
+            return np.reshape(to_reshape, (1, -1))
+
     def get_time_error_z_score(
         self,
         time_weight: float = 1,
         error_weight: float = 1,
     ) -> float:
-
-        if len(self.__time_dataset) > 1:
-            reshaped_time = np.reshape(self.__time_dataset, (-1, 1))
-            reshaped_error = np.reshape(self.__error_dataset, (-1, 1))
-        else:
-            reshaped_time = np.reshape(self.__time_dataset, (1, -1))
-            reshaped_error = np.reshape(self.__error_dataset, (1, -1))
+        reshaped_time = self.__reshape(self.__time_dataset)
+        reshaped_error = self.__reshape(self.__error_dataset)
         self.__scaler.fit(reshaped_time)
         time_z_score: float = self.__scaler.transform(reshaped_time)[-1][0]
         self.__scaler.fit(reshaped_error)
@@ -263,6 +299,58 @@ class Controller(ABC):
         except ValueError:
             return 0
 
+    def compute_first_order_sensitivity_index(self):
+        if len(self.__cost_dataset) == 1:
+            return 0
+        # kernel = gp.kernels.ConstantKernel(1.0, (1e-1, 1e3)) * gp.kernels.RBF(
+        #    10.0, (1e-3, 1e3)
+        # )
+        # model_time = gp.GaussianProcessRegressor(
+        #    kernel=kernel, n_restarts_optimizer=1, alpha=0.1, normalize_y=True
+        # )
+        # model_error = gp.GaussianProcessRegressor(
+        #    kernel=kernel, n_restarts_optimizer=1, alpha=0.1, normalize_y=True
+        # )
+        reshaped_time = self.__reshape(self.__time_dataset)
+        reshaped_error = self.__reshape(self.__error_dataset)
+        self.__time_model.learn_one(
+            ChainedEstimator.X_to_river([self.__time_dataset[-1]]),
+            self.__cost_dataset[-1],
+        )
+        self.__error_model.learn_one(
+            ChainedEstimator.X_to_river([self.__error_dataset[-1]]),
+            self.__cost_dataset[-1],
+        )
+        # model_time.fit(reshaped_time, self.__cost_dataset)
+        # model_error.fit(reshaped_error, self.__cost_dataset)
+        fitted_time = [
+            float(self.__time_model.predict_one(ChainedEstimator.X_to_river([x])))
+            for x in self.__time_dataset
+        ]
+        fitted_error = [
+            float(self.__error_model.predict_one(ChainedEstimator.X_to_river([x])))
+            for x in self.__error_dataset
+        ]
+        time_importance = np.sum(
+            [
+                ((fitted - np.mean(fitted_time)) ** 2)
+                / (cost - np.mean(self.__cost_dataset)) ** 2
+                for fitted, cost in zip(fitted_time, self.__cost_dataset)
+            ]
+        )
+        error_importance = np.sum(
+            [
+                ((fitted - np.mean(fitted_error)) ** 2)
+                / (cost - np.mean(self.__cost_dataset)) ** 2
+                for fitted, cost in zip(fitted_error, self.__cost_dataset)
+            ]
+        )
+        importance_sum = np.sum([time_importance, error_importance])
+        time_importance = time_importance / importance_sum
+        error_importance = error_importance / importance_sum
+        # target (normalized) importance for both is 0.5
+        return np.abs(time_importance - error_importance)
+
     @staticmethod
     def measure_execution_time(
         func: Callable[..., Any], *args, **kwargs
