@@ -2,7 +2,7 @@ from collections import OrderedDict
 from typing import Union, Callable, List, Dict, Any, Optional, Tuple
 from pathlib import Path
 
-from movado.controller import Controller
+from movado.controller import Controller, is_call_exact
 from movado.estimator import Estimator
 from movado.mab_handler import MabHandler
 import numpy as np
@@ -60,7 +60,7 @@ class VotingController(Controller):
 
     def initialize_debug(self):
         Path(self._controller_debug).open("a").write(
-            "Majority_Model_Parameters, Majority_Size, Point, Exec_Time, Error, Mean Weight, Estimation\n"
+            "Majority_Model_Parameters, Majority_Size, Point, Exec_Time, Error, Exact_Estimated_Calls, Mean Weight, Estimation\n"
         )
 
     def learn(
@@ -224,6 +224,10 @@ class VotingController(Controller):
                     "Point": point,
                     "Exec_Time": exec_time,
                     "Error": self._estimator.get_error(),
+                    "Exact_Estimated_Calls": [
+                        is_call_exact.count(True),
+                        is_call_exact.count(False),
+                    ],
                     "Mean Weight": np.mean(
                         [
                             ctrl.get_weight_mab().get_last_action()
@@ -247,6 +251,8 @@ class VotingController(Controller):
             + ", "
             + str(debug_info["Error"])
             + ", "
+            + str(debug_info["Exact_Estimated_Calls"])
+            + ", "
             + str(debug_info["Mean Weight"])
             + ", "
             + str(debug_info["Estimation"])
